#include <LCD5110_Graph.h>
#include <avr/pgmspace.h>
#include <math.h>

LCD5110 myGLCD(9,10,11,12,13);

//digital
#define RPM_LOW 7
#define RPM_HIGH 8
#define INJ_LOW 4
#define VSS_IN 5  //vss signal MUST REMAIN on digital 5 - the only pin with hardware pulse counting

//analog pins
#define MENU_BUTTON 0
#define TEMP 1       
#define READTANK 2   
#define READBATERY 3 


//-------------------------------------- INJECTOR TIME ---------------------------------------
// in my case injectors are 181ml/min - taken from bosch catalogue (XLS file from internet) for my model 
// engine 1.8 16V XU7JP4
// 181ml/min -> 181/60 -> 3.01666666 -> 3.012ml/sec -> /1000 -> 0.003012ml/ms -> if pulse is 2.1ms 
// for example , then -> 0.003012*2.1 -> 0.0063252ml/pulse
// my engine is 2x2 groups injections -> or inject every engine rotation , then again example:
// 900rpm are 900 turns per minute -> /60 -> 15 rotation per second or 15 injections per second per 
// injector group or 15 * 2.1ms -> 31.5ms injector time -> *0.003012ml/ms -> 0.094878 ml * 4 cilinder 
// -> 0.379512 *3600 -> 1366.24 ml/h -> /1000 -> 1.366 l/h
// all variables that change are rpm and injector pulse lenght. 
// Example - In my case: 14.6hz(850rpm) - 105hz(6300rpm) and injector pulses 1.2ms - 8.5ms(about, 
// depend load , ecu and so on) or (0.003012ml/ms * 4 injecotrs work in 1Hour * 3600sec in 1H ) / 1000(1000ml =1L) 
// => 0.0433728 -> 0.0434(constant) * Hz * inject pulse => return car consumtion as L/H
// then based on VSS signal(return calculation as KM/H) we have L/100 consumtion
//
//------------------------------------- VSS speed -------------------------------------------------
// * my car original tyres 185/65/15 -> (185*0.65)*2 + 15*25.4 -> 621.5mm -> 0.6215m * 3.1416 -> 
// 1.9525m for 1 rotation
// * VSS sensor is 22(shaft gear) : 18(sensor gear) or > 1.222 : 1 gear ratio 
// * VSS make 8 pulses for 1 rotation so > 1.9525m / 1.222:1 > 1.5975m for 1 sensor rotation or 
// 1.5975m / 8 pulses > 0.19969m for 1 pulse or if we round - 0.2m/per pulse. I found manuals that mention 5000pulses 
// for 1km of distance and then that 0.2m/pp make sense..i guess 
// If we get 1 pulse/second(1Hz signal) > 0.19969m *3.6 -> 0.7189 m/s speed with 1pulse per second
// In my case i took the tyre lengh as 1.9m Why? Because when it is with load the measure from the tyre center to 
// the ground will not be 0.6215m/2 -> R0.31075m but less(i guess) so if i take 1.9m as tyre lenght and devide to 1.222 
// => 1.5545 -> 1.5545 / 8 -> 0.19432 m/per pulse -> * 3.6  to go to m/s -> 0.69955 < that is my
// variable or if i have 100hz pulses signal * 0.69954 -> 69.954 km/h  If you wish to try, take the VSS signal 
// to some suitable frequency measure device and make 70km/h on tacho and check if you have 100Hz signal.. 
// If not, make based on top info reverce calculation for 50km/h and add that variable in the code.  
//
//----------------------------------- TEMPERATURE SENSOR  ---------------------------------------------
// 5v----10k(1% resistor)---+-----thermistor 4.7k (Y)ellow(V)iolette(R)ed(G)old------GND
//                          |
//                    arduino analog
//
//--------------------------------- Car voltage measure --------------------------------------------------
// 1-20V----[3kOhm]-----+----[1kOhm]---GND (use 1% resistors because of the precision)
//                      |
//                  arduino analog pin
//---------------------------------------------------------------------------------------------------------------------

//---------- Graph symbols ----
//° 8px 
uint8_t degree [] PROGMEM = {0x07, 0x05, 0x07, 0x00};
//° 16pix
uint8_t degree16 [] PROGMEM = {0x0E, 0x11, 0x11, 0x0E, 0x00, 0x00, 0x00, 0x00};
//sign C
uint8_t c16x16 [] PROGMEM = {0xF8, 0xFC, 0x0E, 0x06, 0x06, 0x0E, 0x3C, 0x38, 0x1F, 0x3F, 0x70, 0x60, 0x60, 0x70, 0x3C, 0x1C};
//thermo symbol 8*16
uint8_t termobig [] PROGMEM = {0x24, 0x24, 0xFE, 0xF9, 0x01, 0xFE, 0x24, 0x24, 0x39, 0x7D, 0xFF, 0xFF, 0xF8, 0xF3, 0x45, 0x39};
//batery 16*24px
uint8_t batery1624 [] PROGMEM = {
0x00, 0x00, 0xFC, 0x04, 0x87, 0x87, 0xE7, 0x24, 0xE4, 0x84, 0x84, 0x04, 0x04, 0x04, 0x84, 0x84,
0x84, 0x87, 0x87, 0x87, 0x04, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x40, 0x43, 0x42, 0x4E, 0x48,
0x4E, 0x42, 0x43, 0x40, 0x40, 0x40, 0x43, 0x42, 0x42, 0x42, 0x42, 0x43, 0x40, 0x7F, 0x00, 0x00
};
//------------------------------- floating dot ---------------------------
uint8_t dote [] PROGMEM = {0x1C, 0x22, 0x22, 0x1C};
uint8_t dotf [] PROGMEM = {0x1C, 0x3E, 0x3E, 0x1C}; //4x8px
//------------------------------------------------------------------------

// xantia  logo
uint8_t xantia[] PROGMEM ={
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x3C, 0x78, 0xF0, 0xC0, 0x80, 0x00, 0x00,
0x00, 0x00, 0x00, 0x80, 0x80, 0xC0, 0xC0, 0xE0, 0xE0, 0x70, 0x30, 0x30, 0x38, 0x18, 0x18, 0x18,
0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xE0, 0xF8, 0xBC, 0x9E, 0x86, 0x40, 0x40, 0x00, 0x80,
0xC0, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xC0, 0xC0, 0xE0, 0xF1,
0x73, 0x3F, 0x3E, 0x3E, 0x7E, 0xE7, 0xC7, 0x87, 0x03, 0x01, 0x01, 0x00, 0xC0, 0xE0, 0xF0, 0x38,
0x18, 0x0C, 0x8C, 0xCC, 0x6C, 0xFC, 0xF8, 0x00, 0x00, 0x80, 0xC0, 0x60, 0xF0, 0xFC, 0xFE, 0xC6,
0x60, 0x30, 0x18, 0x18, 0x78, 0xF0, 0xE2, 0x83, 0xC1, 0xFD, 0xFF, 0xFF, 0x83, 0xC0, 0xE0, 0x30,
0x18, 0xFC, 0xFC, 0xFD, 0x8D, 0xC0, 0xE0, 0xF0, 0xF8, 0x9C, 0x8E, 0x86, 0xC6, 0x66, 0x7E, 0xFC,
0x80, 0x80, 0xC0, 0x60, 0x30, 0x18, 0x0C, 0x00, 0x00, 0x60, 0x70, 0x78, 0x3C, 0x1E, 0x0F, 0x07,
0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x07, 0x0E, 0x1C, 0x38,
0x70, 0x61, 0xE3, 0xC3, 0xC3, 0xC3, 0x81, 0x00, 0x00, 0x01, 0x03, 0x03, 0x03, 0x01, 0x00, 0x00,
0x03, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x03, 0x01, 0x00, 0x01, 0x03,
0x03, 0x01, 0x00, 0x00, 0x00, 0x01, 0x03, 0x03, 0x01, 0x00, 0x00, 0x01, 0x03, 0x03, 0x01, 0x01,
0x00, 0x00, 0x00, 0x01, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

//myGLCD.drawBitmap(x, y, zeroHundr, 84, 48);
uint8_t zeroHundr [] PROGMEM = {
0xE0, 0xF0, 0x10, 0xF0, 0xE0, 0x00, 0x80, 0x80, 0x00, 0xE0, 0xF0, 0x00, 0xE0, 0xF0, 0x10, 0xF0,
0xE0, 0x00, 0xE0, 0xF0, 0x10, 0xF0, 0xE0, 0x00, 0x60, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xE0, 0xA0, 0xA0,
0xA0, 0xA0, 0x00, 0x00, 0x03, 0x07, 0x04, 0x07, 0x03, 0x00, 0x01, 0x01, 0x00, 0x07, 0x07, 0x00,
0x03, 0x07, 0x04, 0x07, 0x03, 0x00, 0x03, 0x07, 0x04, 0x07, 0x03, 0x00, 0x06, 0x06, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x02, 0x02, 0x02, 0x02, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xE0, 0x20, 0xE0, 0xC0, 0x00,
0x00, 0x00, 0x00, 0x00, 0xE0, 0xE0, 0xA0, 0xA0, 0x20, 0x00, 0xC0, 0xE0, 0x20, 0xE0, 0xC0, 0x00,
0xC0, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x80, 0xC0, 0x40, 0x40, 0x40, 0x40, 0x00, 0x00, 0x00, 0x00, 0x07, 0x0F,
0x08, 0x0F, 0x07, 0x00, 0x03, 0x03, 0x03, 0x00, 0x08, 0x08, 0x08, 0x0F, 0x07, 0x00, 0x07, 0x0F,
0x08, 0x0F, 0x07, 0x00, 0x0C, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x05, 0x05, 0x05, 0x07, 0x03, 0x00, 0x00,
0x00, 0xE0, 0xE0, 0x00, 0x80, 0xE0, 0x60, 0x00, 0xE0, 0xE0, 0x80, 0x00, 0x80, 0xE0, 0xE0, 0x00,
0xE0, 0xE0, 0x00, 0x00, 0xE0, 0xE0, 0x00, 0x00, 0xC0, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x03, 0x04, 0x1C, 0x18, 0x00, 0x1F, 0x1F, 0x00, 0x03,
0x00, 0x1F, 0x1F, 0x00, 0x1F, 0x1F, 0x01, 0x01, 0x1F, 0x1F, 0x00, 0x00, 0x0C, 0x0C, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};


//battery sign 
uint8_t batery [] PROGMEM = {0xFE, 0x93, 0xBB, 0x92, 0x82, 0x82, 0x92, 0x93, 0x93, 0xFE};
//myGLCD.drawBitmap(26, 0, batery, 10, 8);

// L-H - OK
uint8_t lh[] PROGMEM  = {0x1F, 0x10, 0x90, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0xF9, 0x20, 0x20, 0xF8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
//myGLCD.drawBitmap(26, 0, fuel, 24, 8);

// L-100 -OK 
uint8_t lsto[] PROGMEM  = {0x1F, 0x10, 0x90, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x11, 0x08, 0xFC, 0x00, 0x78, 0x84, 0x84, 0x78, 0x00, 0x78, 0x84, 0x84, 0x78, 0x00, 0x00};
// myGLCD.drawBitmap(26, 0, fuel, 24, 8);
// fuel big OK
uint8_t fuelbigfull[] PROGMEM  = {0x00, 0xFC, 0x02, 0xF2, 0x8A, 0x8A, 0x8A, 0xF2, 0x02, 0xFC, 0x79, 0x1E, 0xFC, 0x78, 0x00, 0x00, 0x80, 0xFF, 0x80, 0x80, 0xBE, 0x8A, 0x8A, 0x80, 0x80, 0xFF, 0xC0, 0x7C, 0x07, 0x00, 0x00, 0x00};
// org uint8_t fuelbig[] PROGMEM  = {0x00, 0xFC, 0x02, 0xFA, 0x0A, 0x0A, 0xFA, 0xE2, 0x02, 0xFC, 0x00, 0xF1, 0x1E, 0xFC, 0xF8, 0x00, 0x80, 0xFF, 0x80, 0x81, 0xBD, 0x95, 0x85, 0x81, 0x80, 0xFF, 0xC0, 0x60, 0x38, 0x0F, 0x00, 0x00};
//myGLCD.drawBitmap(26, 0, fuel, 16, 16);

// tube - rest of fuel
uint8_t tube[] PROGMEM  = {0xEC, 0xF4, 0xB8, 0x78, 0xFC, 0xFE, 0xF9, 0x79, 0xB9, 0xF9, 0xFE, 0xF0, 0x7F, 0xFF, 0xEF, 0xF7, 0xFA, 0xFD, 0xFA, 0xF7, 0xEF, 0xFF, 0xFF, 0x7F}; 
//myGLCD.drawBitmap(26, 0, tube, 12, 16);

uint8_t km[] PROGMEM  = {0xFF, 0x18, 0x14, 0x22, 0xC1, 0x00, 0xFF, 0x02, 0x0C, 0x02, 0xFF}; //show KM
//myGLCD.drawBitmap(26, 0, km, 11, 8);
uint8_t hr[] PROGMEM  = {0xFF, 0x10, 0x10, 0x10, 0xFF, 0x00, 0xFC, 0x24, 0x64, 0xA4, 0x98}; //show LH
//myGLCD.drawBitmap(26, 0, hr, 11, 8); //vertical

uint8_t fi[] PROGMEM  = {0x00, 0x00, 0xF0, 0xF8, 0x0C, 0x8C, 0xCC, 0x6C, 0xFC, 0xF8, 0x0C, 0x06, 0x30, 0x18, 0x0F, 0x1F,0x1B, 0x19, 0x18, 0x18, 0x0F, 0x07, 0x00, 0x00}; 
//myGLCD.drawBitmap(26, 0, fi, 12, 16);

uint8_t RPM24x16[] PROGMEM  = {
0x00, 0xF0, 0xF0, 0x90, 0x90, 0x90, 0xF0, 0x60, 0x00, 0xF0, 0xF0, 0x90, 0x90, 0x90, 0xF0, 0x60,
0x00, 0xF0, 0xF0, 0x60, 0xC0, 0x60, 0xF0, 0xF0, 0x00, 0x03, 0x03, 0x00, 0x01, 0x03, 0x02, 0x02,
0x00, 0x03, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x00, 0x00, 0x00, 0x03, 0x03
};
//myGLCD.drawBitmap(x, y, RPM24x16, 24, 16);

uint8_t kmh24x16[] PROGMEM = {
0x00, 0x7E, 0x7E, 0x10, 0x3C, 0x6E, 0x42, 0x00, 0x00, 0x7E, 0x7E, 0x04, 0x18, 0x04, 0x7E, 0x7E,
0x00, 0x80, 0x40, 0x20, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01, 0x00, 0x7E, 0x7E, 0x10, 0x10, 0x7E, 0x7E
};
//myGLCD.drawBitmap(x, y, kmh24x16, 24, 16);

//------ 2 pictures - empty tank animation ----
uint8_t empty1[] PROGMEM  = {0x00, 0xFC, 0x02, 0xF2, 0x8A, 0x8A, 0x8A, 0xF2, 0x02, 0xFC, 0x00, 0x00, 0xC8, 0xF0, 0xE0, 0xC0, 0x80, 0xFF, 0x80, 0x80, 0xBE, 0xAA, 0xAA, 0x80, 0x80, 0xFF, 0xC0, 0x40, 0x43, 0x20, 0x1F, 0x03};
uint8_t empty2[] PROGMEM  = {0x00, 0xFC, 0x02, 0xF2, 0x8A, 0x8A, 0x8A, 0xF2, 0x02, 0xFC, 0x00, 0x00, 0xF2, 0x3C, 0xF8, 0xF0, 0x80, 0xFF, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0xFF, 0xC0, 0x40, 0x40, 0x20, 0x1F, 0x00};


extern uint8_t MediumNumbers[]; // my font
extern uint8_t SmallFont[]; // my font

boolean emptyFlip,okFifth,okHund;
int rpm, kmh, menu, tmpTimer, minGone;
long int tank,valTnk; 
int tmpTPS, tmpMap, count;
float inject, vss,volTs, tankSplash, litrHrCntr,rpmHz,ftemp;
float consump,tmpCons,consumSto,tmpTank,xb,xc,zero2fifthy,zero2hundr;
long int tmpRpmLow, tmpRpmHigh;
long int tmpInjLow, tmpVss, tmpVssL, tmpVssH; 
long int lastmillis,butonMicroPrev,timer1sec, tankTimer, ziroHundr;
byte zeroHundred;
//------------------------------------------------------ themperature variables -----------------------------------
unsigned long tmperOld;
int  raw = 0;
float vOut = 0;        // variable to store the output voltage
float r2 = 0;          // variable to store the R2 value
float buffer = 0;      // buffer variable for calculation
// from data sheet of thermistor
float a1=0.003354016;
float b1=0.000256985;
float c1=0.000002620131;
float d1=0.00000006383091;
float tcels=0; //final show temp
//---------------------------------------------------------------------------------------------------------------- 
unsigned int getCount(){
TCCR1B= 0 ; // Gate Off / Counter Tn stopped
count = TCNT1;
TCNT1 = 0;
bitSet(TCCR1B ,CS12); // Counter Clock source is external pin
bitSet(TCCR1B ,CS11); // Clock on rising edge
//bitSet(TCCR1B ,CS10); // you can clear this bit for falling edge if On(1) the is on rising edge
return count;
}
//---------------------------------------------------------------------------------------------------------------- 


void setup()
{
//--------------------------------------------------------------------------------------- 
  myGLCD.InitLCD(58); // set contrast 
  
  digitalWrite(VSS_IN, LOW);// hardware counter setup (see ATmega data sheet for details)
  TCCR1A=0; // reset timer/counter control register A
  getCount(); // this will start the clock

      pinMode(RPM_LOW, INPUT); 
 digitalWrite(RPM_LOW,LOW);
      pinMode(RPM_HIGH, INPUT);
 digitalWrite(RPM_HIGH,LOW);  
      pinMode(INJ_LOW, INPUT);
 digitalWrite(INJ_LOW,LOW);  

  myGLCD.drawBitmap(0, 0, xantia, 84, 48);
  myGLCD.setFont(SmallFont);
  myGLCD.print("1.8 16V - 1999", 0, 29);
  myGLCD.print("ECU:GAGEM SL96", 0, 37);
  myGLCD.update();
  delay(2000); 
  
  termo();  //read  temp. resistor
  delay(10);
  battery(); // read battery voltage
  delay(10);
  
  myGLCD.clrScr();    
  myGLCD.update();
 
  printTermo();
  printBatery();
  
  myGLCD.setFont(SmallFont);
  myGLCD.print("XU7JP4-1.8 16V", 0, 41);
  myGLCD.update();
 

   for(int ax=0; ax < 84 ; ax+=5){
       myGLCD.drawBitmap(ax,33, dote, 4, 8);
       myGLCD.update();
   }
   for(int fx=0; fx<84; fx+=5){
       myGLCD.drawBitmap(fx,33, dotf, 4, 8);
       myGLCD.update();
       delay(300);
   }
  
  myGLCD.clrScr();    
  myGLCD.update();

  menu = 1;
  tankFunct();     
  tankSplash = tank; //read tank level and assign it to variable for independent calculation
  minGone=0; // reset one minute counter
}


void loop() {
tmpRpmLow=pulseIn(RPM_LOW, LOW);
tmpRpmHigh=pulseIn(RPM_HIGH, HIGH);
tmpInjLow=pulseIn(INJ_LOW, LOW,300000);
//------------------------------------------------------ timer of about 1 sec -----------------------------
if(millis() - timer1sec >= 980){ //with that value only all work correct, i down`t know ... why :)
 
     //*********************** 
      tmpVss = getCount(); //get counted pulses on pin Digital 5  
     //*********************** 
     tmpTimer ++;
      minGone ++; 
           if(tmpTimer > 5){ //-- read around every 6 seconds
              tmpTimer = 0;
                   termo(); // read temperature
                 battery(); // read battery voltage
              litrHrCntr = litrHrCntr+consump;
              
           }     
      if(minGone > 59){
            float lhtmp = (litrHrCntr/10.0)/60.0;
            tankSplash = tankSplash - lhtmp;
            minGone = 0;
            litrHrCntr = 0;
            tankTimer ++ ;
      }  

// because the tank liquid is splashing during driving the car dash take instant reading on stand still position
// and based on rpm, temp. slowly recalculate the level of the fuel + probably from time to time level check
// every 6min (about) that formula will check if the tank value has change. For example if you stop the car sideway
// or on the hill, the reading will be wrong. So if there appear more than 2L difference on absolute value it will take the new tank 
// reading and will continue from there. Exampl.: 
// tankSplash(12L) - tank(10L) = 2L -> abs 2 -> 2 
// tankSplash(10L) - tank(14L) = -4L -> abs -4 -> 4 

          if(tankTimer > 5){
               tankFunct(); //check again tank level    
               int tmpTnk= tankSplash - tank;
                   if(abs(tmpTnk) > 4){
                     tankSplash = tank;
                   }
               tankTimer = 0;
          }

     emptyFlip = !emptyFlip; //responsible for empty fuel animation, appear when fuel is below 10L  
     timer1sec = millis();
}
//------------------------------------
int long xa = tmpRpmHigh+tmpRpmLow;
if(xa != 0){
         xb = xa/1000.0; 
         rpmHz = 500.0/xb;//should be 1000/xb but because on idle i have 29Hz signal insted 14.5 @ 870rpm  i take half of freq.That is on my XU7JP4 motor!
         rpm = rpmHz*60;
}else{
         rpm = 0;
}       
//----------------------------------- injector pulse to ms --------------
if(tmpInjLow != 0){
     inject = tmpInjLow/1000.0;
     //inject = inject - 0.4; // error correction - if the LCD shows 3.0ms on lexia it shows 2.5-2.6ms | remove comment if you wish to use
}else{
     inject = 0;
}     
//----------------------- vss / km/h calc ------------------------------
if(tmpVss !=0){
   float tmpkMh = (tmpVss*0.69954)+0.5;// calc km/h 
   kmh = tmpkMh;
}else{
   kmh=0;
}   
//------------------------------ fuel consump --------------------------
if(inject != 0){
    consump = rpmHz*inject*0.0435; //return in L/H
}else{
    consump = 0;
}  
//*************
    if((kmh > 3)&&(consump !=0)){ //-- if the car moving and engine work -> show L/100
         consumSto = (consump*100.0)/kmh; 
         
          if(consumSto > 50){
                consumSto = 50;
          }
         tmpTank = (tankSplash*100.0)/consumSto; 
    }
//*************
    if((kmh <= 3)&&(consump !=0)){ //-- if the car not moving and engine work -> show L/H
        consumSto = consump;
        tmpTank = tankSplash/consump;  
    }
//*************
    if((kmh > 3)&&(consump ==0)){  //-- if the car moving and engine not work engine breaking mode , appear on engine over 2000rpm without throttle 
        consump = 0;
        consumSto = 0;
        tmpTank = (tankSplash*100)/1.5;
    }
    
//----------------------------------------------------- read for menu change ---------------------------------
if(millis() - butonMicroPrev > 350 ){
   if(analogRead(MENU_BUTTON) > 150)
     {
        myGLCD.clrScr();
        myGLCD.update(); 
        menu +=1;
           if(menu > 6){
               menu = 1;
           }
     }
     butonMicroPrev = millis();
}       


//  +--------------------------------------------------+
//  |       print data with timeout as millis          |
//  +--------------------------------------------------+

if(menu==1){//---------------------------------------------------- menu 1 -----------------------

   if (millis() - lastmillis > 249) //124-original
   { 

     myGLCD.setFont(MediumNumbers);
     myGLCD.drawBitmap(0, 17, fi, 12, 16);
     myGLCD.drawBitmap(0, 32, tube, 12, 16);
     
     myGLCD.print(",,,,,,", 16, 16);   // printing ",,," in big number produce blank space.When design the font i add it.
     myGLCD.printNumF(consumSto, 1, 16, 16); 
     
     myGLCD.print(",,", 18, 0); 
     myGLCD.printNumI(tankSplash, 18, 0); // tank fuel - was before tank
     
     myGLCD.print(",,,,,,", 16, 32); 
     myGLCD.printNumI(tmpTank, 16, 32); // rest km or hrs
     
     if(kmh > 3){
             myGLCD.drawBitmap(61, 20, lsto, 24, 8);
             myGLCD.drawBitmap(67, 35, km, 11, 8);
     }
     if(kmh < 3){
              myGLCD.drawBitmap(61, 20, lh, 24, 8);
              myGLCD.drawBitmap(67, 35, hr, 11, 8); 
     }
     if(tankSplash < 11){
             if(emptyFlip != true){  
                          myGLCD.drawBitmap(0, 0, empty1, 16, 16);
             }else{       
                          myGLCD.drawBitmap(0, 0, empty2, 16, 16);
             }
     }else{
              myGLCD.drawBitmap(0, 0, fuelbigfull, 16, 16); 
     }
      myGLCD.update(); 
    //*******************************************  car voltage&temperature print ******************************* 
      myGLCD.setFont(SmallFont);
      myGLCD.drawBitmap(45, 0, batery, 10, 8); //batery icon
    if(volTs <10){
        myGLCD.print("0",56, 0); 
        myGLCD.printNumF(volTs,1, 62, 0); 
    }else{
        myGLCD.printNumF(volTs,1, 56, 0); 
    }
    //long int volts = (val*1000)/2046;
    //long int volts = vlt/2046; // (volt/1023)*5 -> 1023/5 -> 204.6 -> *10 -> 2046
    //float volts  = (val/1023.0)*5.0;  original
    //myGLCD.printNumF(volts,1, 50, 0); 
    //myGLCD.print("12.5", 58, 0); -- for test show
    
        if(tcels < 0){
                myGLCD.print("-",50, 8);      
        }else{
                myGLCD.print(" ",50, 8);
        }

     ftemp = abs(tcels);        
         if(ftemp < 10){
              myGLCD.print(" ",56, 8);       
              myGLCD.printNumF(ftemp,1,62, 8); 
         }else{
              myGLCD.printNumF(ftemp,1,56, 8);  
         }  
           
    myGLCD.drawBitmap(81, 8, degree, 4, 8);
    myGLCD.update();

    //------------ end timer ------------- 
     lastmillis = millis();
    }//end timer - last mills
}


if(menu==2){//----------------------------------------------- menu 2 ------ temperature rpm kmh ----------------

  if (millis() - lastmillis > 249) //124-original
  { 
     printTermo();
     myGLCD.drawBitmap(0, 16, RPM24x16, 24, 16);
     myGLCD.drawBitmap(0, 32, kmh24x16, 24, 16);
     
     myGLCD.print(",,,,", 28, 16);   
     myGLCD.print(",,,,", 28, 32);   
     
     myGLCD.printNumI(rpm, 28, 16); 
     myGLCD.printNumI(kmh, 28, 32); 
     myGLCD.update(); 
    //------------ end timer ------------- 
    lastmillis = millis();
   }//end timer - last mills

}

if(menu==3){//----------------------------------------------- menu 3 ---- 0-100 ------------------

   if(kmh <= 0){
        zeroHundred = B000; //bit2. -> enabler,  bit.1 -> 100km/h , bit.0 -> 50km/h
        zero2fifthy=zero2hundr=0;
   }
   if(kmh > 0 && kmh < 3){
        ziroHundr=millis();
        zeroHundred = B111;
   }         
   if(kmh < 51 && bitRead(zeroHundred,0)==1){
        zero2fifthy = xc;
     }else{
        bitWrite(zeroHundred,0,0);
   }
   if(kmh < 101 && bitRead(zeroHundred,1)==1){
         zero2hundr = xc;       
     }else{
        bitWrite(zeroHundred,1,0);
        bitWrite(zeroHundred,2,0);
   }

 myGLCD.setFont(MediumNumbers);
 myGLCD.drawBitmap(0, 0, zeroHundr, 84, 48);
 myGLCD.printNumF(zero2hundr,1, 28, 0);
 myGLCD.printNumF(zero2fifthy,1, 28, 17);
 myGLCD.printNumI(kmh, 28, 33); 

if (millis() - lastmillis > 99){   //count and update every 0.1 sec   
     if(bitRead(zeroHundred,2)==1){
       xc = (millis()-ziroHundr)/1000.0;
     }
 
 myGLCD.update();  
  //------------ end timer ------------- 
 lastmillis = millis();
 }//end timer - last mills

}// end menu 3


if(menu==4){//----------------------------------------------- menu 4 --------- debug -------------

     if (millis() - lastmillis > 249) //124-original
     { 
      myGLCD.setFont(SmallFont);
      myGLCD.print("injMs:", 0, 0);
      myGLCD.print("    ", 36, 0); 
      myGLCD.printNumF(inject ,1,36, 0); 

      myGLCD.print("rpmMs:", 0, 10);
      myGLCD.print("    ", 36, 10); 
      myGLCD.printNumF(xb,1,36, 10); 

      myGLCD.print("vssMs:", 0, 20);
      myGLCD.print("    ", 36, 20); 
      myGLCD.printNumF(vss,1,36,20); 

      myGLCD.print("RPM:", 0, 30);
      myGLCD.print("    ", 24, 30); 
      myGLCD.printNumI(rpm,24,30); 

      myGLCD.print("L/H:", 0, 40);
      myGLCD.print("    ", 24, 40); 
      myGLCD.printNumF(consump,1,24,40); 

      myGLCD.print("KH:", 50, 40);
      myGLCD.print("    ", 67, 40); 
      myGLCD.printNumI(kmh,67,40); 
      myGLCD.update(); 

      //------------ end timer ------------- 
      lastmillis = millis();
      }//end timer - last mills

}


if(menu==5){//--------------------------------------------------- menu 5---------debug --------------

    if (millis() - lastmillis > 249) //124-original
    { 
      myGLCD.setFont(SmallFont);
      myGLCD.print("RPM:", 0, 0);
      myGLCD.print("    ", 25, 00); 
      myGLCD.printNumI(rpm,25,00);
     
      myGLCD.print("|", 49, 0);
      myGLCD.print("|", 49, 10);
     
     
      myGLCD.print("rpmHz", 55, 0);
      myGLCD.print("     ", 55, 10); 
      myGLCD.printNumF(rpmHz,1,55,10); 
      
     
      myGLCD.print("L/H:", 0, 10);
      myGLCD.print("    ", 25, 10); 
      myGLCD.printNumF(consump,1,25,10); 
   
      
      myGLCD.print("KMH:", 0, 20);
      myGLCD.print("    ", 25, 20); 
      myGLCD.printNumI(kmh,25,20); 

      if(kmh !=0){
        tmpCons = (consump*100.0)/(float)kmh; 
      }else{
        tmpCons = 0;
      }   
      myGLCD.print("L/100:", 0, 30);
      myGLCD.print("    ", 38, 30); 
      myGLCD.printNumF(tmpCons,1,38, 30); 
      
      myGLCD.print("injMs:", 0, 40);
      myGLCD.print("    ", 38, 40); 
      myGLCD.printNumF(inject ,1,38, 40);
      myGLCD.update(); 

     //------------ end timer ------------- 
      lastmillis = millis();
     }//end timer - last mills      
      
}

if(menu==6){//--------------------------------------------------- menu 6 -----------------------
 
   if (millis() - lastmillis > 499) //124-original
   { 
 
      myGLCD.setFont(SmallFont);
      myGLCD.print("TANK:", 0, 0);
      myGLCD.print("    ", 32, 0); 
      myGLCD.printNumF(tankSplash,2,32,0); 

      myGLCD.print(" BAT:", 0, 10);
      myGLCD.print("    ", 32, 10); 
      myGLCD.printNumF(volTs,1,32,10); 

      myGLCD.print("TEMP:", 0, 20);
      myGLCD.print("    ", 32, 20); 
      myGLCD.printNumF(tcels,1,32,20); 

      myGLCD.print("1/6:", 0, 30);
      myGLCD.print(" ", 25, 30); 
      myGLCD.printNumI(tmpTimer,25,30); 

      myGLCD.print("sec:", 40, 30);
      myGLCD.print("  ", 63, 30); 
      myGLCD.printNumI(minGone,63,30); 
      myGLCD.update(); 
      //------------ end timer ------------- 
      lastmillis = millis();
    }//end timer - last mills
}
  
 
}// end LOOP  
  
  
//----------------------------------------------------------------------------------------------------
void tankFunct(){
    valTnk = analogRead(READTANK);
    float tmpFtank = 0.11875*(float)valTnk;
    tank =78-tmpFtank;  // tank resistor measure 50 oms full to 300oms empty
}
//----------------------------------------------------------------------------------------------------  
void battery(){
    int val = analogRead(READBATERY); //read batery voltage -> for test now 3.3V from arduino
    volTs = (float)val*0.0197; // (val/256)*5v for: 3kOhm ---+----1kOhm or 0-20V measure then 5V/256 -> 0.0196xxx
}  
//----------------------------------------------------------------------------------------------------
void termo(){
   raw = analogRead(TEMP);    // Reads the Input PIN
   vOut = 0.0048828 * raw;    // Calculates the Voltage on th Input PIN  - (5.0 / 1024.0) * raw
   buffer = (5 / vOut) - 1;
   r2 = 10 / buffer;  //get resistance of thermistor and calculate based on formula for that kind of thermistor
  
   float ln = log(r2/4.7);
   float ln1 = b1*ln;
   float ln2 = c1*ln*ln;
   float ln3 = d1*ln*ln*ln;
   float delta = 1/(a1+ln1+ln2+ln3);
   tcels = delta -273.15;
 }
 //----------------------------------------------------------------------------------------------------
void printTermo(){
   
   myGLCD.setFont(MediumNumbers);
   myGLCD.drawBitmap(0, 0, termobig, 8, 16); //big termo symbol
 
   if(tcels < 0){
            myGLCD.print("-",9, 0);
   }else{
            myGLCD.print(",",9, 0);
   }

   float tv = abs(tcels);
   if(tv < 10){
            myGLCD.print("0",17, 0);
            myGLCD.printNumF(tv,1,27, 0);
   }else{
            myGLCD.printNumF(tv,1,17, 0);
   }
//myGLCD.drawBitmap(0, 20, degree, 4, 8);  //small font degree
   myGLCD.drawBitmap(60, 0, degree16, 4, 16);
   myGLCD.drawBitmap(65, 0, c16x16, 8, 16);
   myGLCD.update();
}
//----------------------------------------------------------------------------------------------------  
void printBatery(){
   myGLCD.setFont(MediumNumbers);
   myGLCD.drawBitmap(0, 17, batery1624, 24, 16);
   myGLCD.printNumF(volTs,1, 26, 17);
   myGLCD.update();
}  
